# -*- coding: utf8 -*-
import logging
from os.path import join

import yaml
from frigg_test_discovery import detect_test_tasks

logger = logging.getLogger(__name__)


def build_tasks(directory, docker):
    return detect_test_tasks(docker.list_files(directory))


def load_settings_file(path, docker):
    return yaml.load(docker.read_file(path))


def get_path_of_settings_file(directory, docker):
    if docker.file_exist(join(directory, '.frigg.yml')):
        return join(directory, '.frigg.yml')
    elif docker.file_exist(join(directory, '.frigg.yaml')):
        return join(directory, '.frigg.yaml')


def build_settings(directory, docker):
    path = get_path_of_settings_file(directory, docker)

    settings = {
        'webhooks': [],
        'services': []
    }

    if path is not None:
        settings.update(load_settings_file(path, docker))
    else:
        settings['tasks'] = build_tasks(directory, docker)

    if len(settings['tasks']) == 0:
        raise RuntimeError('No tasks found')

    return settings


class CachedProperty(object):
    def __init__(self, func, name=None):
        self.func = func
        self.__doc__ = getattr(func, '__doc__')
        self.name = name or func.__name__

    def __get__(self, instance, type=None):
        if instance is None:
            return self
        res = instance.__dict__[self.name] = self.func(instance)
        return res


cached_property = CachedProperty



# -*- coding: utf8 -*-
import json
import logging
import random
import socket
import time

import requests
from docker.manager import Docker

from .jobs import Build

logger = logging.getLogger(__name__)


def fetcher(**options):
    notify_of_upstart(options)
    while options['dispatcher_url']:
        task = fetch_task(options['dispatcher_url'], options['dispatcher_token'])
        if task:
            start_build(task, options)

        time.sleep(5.0 + random.randint(1, 100) / 100)


def start_build(task, options):
    docker_options = {
        'image': options['docker_image'],
        'combine_outputs': True,
        'privilege': True
    }
    with Docker(**docker_options) as docker:
        try:
            build = Build(task['id'], task, docker=docker, worker_options=options)
            logger.info('Starting {0}'.format(task))
            build.run_tests()
        except Exception as e:
            logger.exception(e)


def fetch_task(dispatcher_url, dispatcher_token):
    logger.debug('Fetching new job from {0}'.format(dispatcher_url))
    response = requests.get(
        dispatcher_url,
        headers={
            'x-frigg-worker-token': dispatcher_token
        }
    )

    if response.status_code == 200:
        return response.json()['job']
    else:
        time.sleep(20)


def notify_of_upstart(options):
    if options['slack_url']:
        host = socket.gethostname()
        requests.post(options['slack_url'], json.dumps({
            'icon_emoji': options['slack_icon'],
            'channel': options['slack_channel'],
            'text': 'I just started on host {host}, looking for work now...'.format(host=host),
            'username': 'frigg-worker',
        }))



# -*- coding: utf-8 -*-
import logging

import requests

logger = logging.getLogger(__name__)


class APIWrapper(object):

    def __init__(self, options):
        self.token = options['hq_token']
        self.url = options['hq_url']

    @property
    def headers(self):
        return {
            'content-type': 'application/json',
            'FRIGG_WORKER_TOKEN': self.token
        }

    def get(self, url):
        return requests.post(url, headers=self.headers)

    def post(self, url, data):
        return requests.post(url, data=data, headers=self.headers)

    def report_run(self, build_id, build):
        response = self.post(self.url, data=build)
        logger.info('Reported build to hq, hq response status-code: {0}, data:\n{1}'.format(
            response.status_code,
            build
        ))
        if response.status_code != 200:
            logger.error('Report of build failed, response status-code: {0}, data:\n{1}'.format(
                response.status_code,
                build
            ))
            with open('build-{0}-hq-response.html'.format(build_id), 'w') as f:
                f.write(response.text)
        return response



# -*- encoding: utf8 -*-
import codecs
from os import path
from setuptools import setup, find_packages


def read(*parts):
    file_path = path.join(path.dirname(__file__), *parts)
    return codecs.open(file_path, encoding='utf-8').read()

setup(
    name='frigg-worker',
    version='1.0.1',
    description='A worker application that listens to the frigg broker '
                'an pick up builds and build them.',
    long_description=read('README.rst'),
    author='The frigg team',
    author_email='hi@frigg.io',
    license='MIT',
    url='https://github.com/frigg/frigg-worker',
    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    install_requires=[
        'click==4.0',
        'frigg-coverage==1.0.0',
        'frigg-test-discovery>0.0,<1.1',
        'docker-wrapper==0.5.0',
        'pyyaml==3.11',
        'raven==5.2.0'
    ],
    entry_points={
        'console_scripts': ['frigg-worker = frigg_worker.cli:start']
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3',
    ]
)
